@@ -100,6 +100,12 @@ enum btrfs_read_sb_flags {
 	 * SBREAD_RECOVER
 	 */
 	SBREAD_TEMPORARY = (1 << 1),
+
+	/*
+	 * Equivalent of OPEN_CTREE_IGNORE_FSID_MISMATCH, allow to read
+	 * superblock that has mismatched sb::fsid and sb::dev_item.fsid
+	 */
+	SBREAD_IGNORE_FSID_MISMATCH = (1 << 2),
 };
 
 /*
@@ -131,8 +137,7 @@ struct extent_buffer* btrfs_find_create_tree_block(
 
 void btrfs_setup_root(struct btrfs_root *root, struct btrfs_fs_info *fs_info,
 		      u64 objectid);
-int clean_tree_block(struct btrfs_trans_handle *trans,
-		     struct btrfs_root *root, struct extent_buffer *buf);
+int clean_tree_block(struct extent_buffer *buf);
 
 void btrfs_free_fs_info(struct btrfs_fs_info *fs_info);
 struct btrfs_fs_info *btrfs_new_fs_info(int writable, u64 sb_bytenr);
@@ -165,8 +170,7 @@ static inline int close_ctree(struct btrfs_root *root)
 }
 
 int write_all_supers(struct btrfs_fs_info *fs_info);
-int write_ctree_super(struct btrfs_trans_handle *trans,
-		      struct btrfs_fs_info *fs_info);
+int write_ctree_super(struct btrfs_trans_handle *trans);
 int btrfs_read_dev_super(int fd, struct btrfs_super_block *sb, u64 sb_bytenr,
 		unsigned sbflags);
 int btrfs_map_bh_to_logical(struct btrfs_root *root, struct extent_buffer *bh,
